package uibk.mtk.draw2d.axes2d;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Stroke;
import java.awt.image.BufferedImage;
import java.text.DecimalFormat;
import java.util.Vector;

import javax.swing.UIManager;

import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.geom.geom2d.CoordinateRect2D;
import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.lang.PrepaintComputable;
import uibk.mtk.math.Rounder;
import uibk.mtk.text.Formatter;

/**
 * Achsen f?r ein zweidimensionales Koordinaten
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class Axes2D extends Drawable2D implements PrepaintComputable {

	/** "Stil" der Achsen: Box oder normal */
	int style = MODE_INTELLIGENT;

	/** "Richtung" der Achsen: normal */
	int directionx = 3;

	/** "Richtung" der Achsen: normal */
	int directiony = 3;

	/** "normaler" Achsen-Stil */
	public static final int MODE_BOX = 0;

	/** Box Achsen-Stil */
	public static final int MODE_INTELLIGENT = 1;

	/** Farbe der y-Achse */
	Color colorxaxes = Color.black;

	/** Farbe der x-Achse */
	Color coloryaxes = Color.black;

	/** Linienstil f?r die x-Achse */
	Stroke strokexaxes = new BasicStroke((float) 1.5);

	/** Linienstil f?r die y-Achse */
	Stroke strokeyaxes = new BasicStroke((float) 1.5);

	/** Zahlenformat f?r die x-Achse */
	DecimalFormat formatx;

	/** Zahlenformat f?r die y-Achse */
	DecimalFormat formaty;

	/** Stil zum Zeichnen der Achsen werden mit Ticks aber ohne Beschriftung */
	public static final int AXES_STYLE_TICKS = 0;

	/** Stil zum Zeichnen der Achsen werden mit Ticks und Beschriftung */
	public static final int AXES_STYLE_TICKS_AND_LABELS = 1;

	/** Stil zum Zeichnen der Achsen ohne Ticks und ohne Beschriftung */
	public static final int AXES_STYLE_NOTICKS = 2;

	/** Stil zum Zeichnen der Achsen ohne Ticks und mit Beschriftung */
	public static final int AXES_STYLE_NOTICKS_AND_LABELS = 3;

	/** verwendete Schrift */
	Font font;

	/** Bereich der Ebene die das Koordinatensystem enth?lt */
	int area;

	/** Die x-Achse */

	private XAxis xaxis = new XAxis(this);

	/** Die y-Achse */
	private YAxis yaxis = new YAxis(this);

	/** Scihtbarkeit der x-Achse */
	private boolean visibilityXAxis = true;

	/** Sichtbarkeit der y-Achse */
	private boolean visibilityYAxis = true;

	/** Anzahl der Ticks pro Achse */
	private int numTicks;

	/** Schnittpunkt der Achsen */
	private Punkt2D pointAxesIntersect;

	/** setzt das <CODE>DecimalFormat</CODE> f?r die x-Achse */
	public void setDecimalFormatXAxes(DecimalFormat df) {
		formatx = df;
	}

	/** setzt das <CODE>DecimalFormat</CODE> f?r die y-Achse */
	public void setDecimalFormatYAxes(DecimalFormat df) {
		formaty = df;
	}

	/** Eigenes Format falls kein DecimalFormat gesetzt worden ist */
	private Formatter formatter = new Formatter();

	/** Farbe f?r den Rand falls die Achsen als Box gezeichnet werden */
	private Color edgecolor = UIManager.getColor("Panel.background"); //$NON-NLS-1$

	/** Hilfsklasse f?r die Positionen der Ticks */
	private static class TickPosition {

		/** Position des Ticks auf der Achse */
		double position;

		/** Haupttick ja/nein */
		boolean maintick;

		TickPosition(double position, boolean bigTick) {
			this.position = position;
			this.maintick = bigTick;

		}
	}

	/** Erzeugt die Achsen */
	public Axes2D() {

		setStyleXAxis(AXES_STYLE_TICKS_AND_LABELS);
		setStyleYAxis(AXES_STYLE_TICKS_AND_LABELS);

		this.numTicks = 11;
	}

	/** setzt den Stil der Achsen */
	public void setAxesStyle(int style) {
		this.style = style;
	}

	/** gibt den Stil der Achsen zur?ck */
	public int getAxesStyle() {
		return style;
	}

	/**
	 * Setzt die Sichtbatkeit der x-Achse
	 * 
	 */
	public void setVisibilityXAxis(boolean visible) {
		this.visibilityXAxis = visible;
	}

	/**
	 * Setzt die Sichtbatkeit der y-Achse
	 */
	public void setVisibilityYAxis(boolean visible) {
		this.visibilityYAxis = visible;
	}

	/** setzt den Stil f?r die x-Achse fest */
	public void setStyleXAxis(int style) {
		xaxis.setStyle(style);

	}

	/** setzt den Stil f?r die x-Achse fest */
	public void setDirectionXAxis(int style) {
		directionx = style;
	}

	/** setzt den Stil f?r die x-Achse fest */
	public void setDirectionYAxis(int style) {
		directiony = style;
	}

	/** setzt den Stil f?r die y-Achse fest */
	public void setStyleYAxis(int style) {
		yaxis.setStyle(style);
	}

	/** setzt die Anzahl der Ticks fest */
	public void setNumTicks(int numTicks) {
		this.numTicks = numTicks;
	}

	/** Setzt die zu verwendende Schriftart */
	public void setFont(Font font) {
		this.font = font;
	}

	/**
	 * setzt die Farbe f?r den Rand, die verwendet wird falls die Achsen als Box
	 * gezeichnet werden
	 * 
	 */
	public void setEdgeColor(Color color) {
		edgecolor = color;
	}

	/** F?rbt den Rand um die Achsenbox mit der <CODE>Farbe</CODE> edgecolor */
	private void colorEdge(Graphics2D g2) {

		// Farbe setzen
		g2.setColor(edgecolor);

		// Insets und Rand
		Insets insets = scene2d.getInsets();
		int edge = scene2d.getEdge();

		// Rand links
		g2.fillRect(0, 0, insets.left + edge, panel.getHeight());
		// Rand oben
		g2.fillRect(0, 0, panel.getWidth(), insets.top + edge);
		// Rand unten
		g2.fillRect(0, panel.getHeight() - insets.top - edge, panel.getWidth(),
				insets.top + edge);
		// Rand rechts
		g2.fillRect(panel.getWidth() - edge - insets.right, 0,
				panel.getWidth(), panel.getHeight());

	}

	/** Zeichnet die Achsen mit den Ticks deren Beschriftung und Titel */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {

		// falls BOX-Stil -> Zeichnen des Randes
		if (style == MODE_BOX)
			colorEdge(g2);

		// Zeichnen der x-Achse
		if (xAxisVisible()) {
			xaxis.draw(g2, scene2d);
		}

		// Zeichnen der y-Achse
		if (yAxisVisible()) {
			yaxis.draw(g2, scene2d);
		}

		// falls BOX-Stil -> Zeichnen eines schwarzen Randes um die Achsenbox
		if (style == Axes2D.MODE_BOX) {
			g2.setColor(Color.black);
			Stroke oldStroke = g2.getStroke();
			g2.setStroke(new BasicStroke((float) 1.5));

			int xmin = scene2d.xToPixel(scene2d.getXmin());
			int xmax = scene2d.xToPixel(scene2d.getXmax());
			int ymin = scene2d.yToPixel(scene2d.getYmin());
			int ymax = scene2d.yToPixel(scene2d.getYmax());
			g2.drawLine(xmin, ymin, xmax, ymin);
			g2.drawLine(xmin, ymin, xmin, ymax);
			g2.drawLine(xmax, ymin, xmax, ymax);
			g2.drawLine(xmin, ymax, xmax, ymax);

			g2.setStroke(oldStroke);
		}

	}

	/** gibt zur?ck ob die x-Achse sichtbar ist */
	private boolean xAxisVisible() {

		return visibilityXAxis;

	}

	/** gibt zur?ck ob die y-Achse sichtbar ist */
	private boolean yAxisVisible() {

		return visibilityYAxis;

	}

	/** Vorabberechnung der Achsen */
	public void prepaintcompute() {

		double xmax = scene2d.getXmax();
		double xmin = scene2d.getXmin();
		double ymax = scene2d.getYmax();
		double ymin = scene2d.getYmin();

		// Schnittpunkt der Achsen
		pointAxesIntersect = new Punkt2D();
		// Bereich der Achsen
		area = scene2d.getCoordinates().getArea();
		// Startpunkt x-Achse
		Punkt2D pointXAxesStart = new Punkt2D();
		// Endpunkt x-Achse
		Punkt2D pointXAxesEnd = new Punkt2D();
		// Startpunkt y-Achse
		Punkt2D pointYAxesStart = new Punkt2D();
		// Endpunkt y-Achse
		Punkt2D pointYAxesEnd = new Punkt2D();
		// Position der Ticks auf der x-Achse (TL_BOTTOM,...)
		int tickPosX = 0;
		// Position der Ticks auf der y-Achse (TL_BOTTOM,...)
		int tickPosY = 0;

		// Falls Box-Stil behandeln den Rest als ob AREA_UPPERRIGHT
		// d.h. x-Achse unten und y-Achse links
		if (style == MODE_BOX)
			area = CoordinateRect2D.AREA_UPPERRIGHT;

		// Abh?ngig von den aktuellen Koordinaten:
		// Position der Achsen w?hlen, Schnittpunkt der Achsen setzen
		switch (area) {

		case CoordinateRect2D.AREA_UPPERRIGHT:
			// x-Achse unten mit Beschriftung unten
			// y-Achse links mit Beschriftung links
			pointXAxesStart.x = xmin;
			pointXAxesStart.y = ymin;
			pointXAxesEnd.x = xmax;
			pointXAxesEnd.y = ymin;
			pointYAxesStart.x = xmin;
			pointYAxesStart.y = ymin;
			pointYAxesEnd.x = xmin;
			pointYAxesEnd.y = ymax;
			pointAxesIntersect.x = xmin;
			pointAxesIntersect.y = ymin;
			tickPosX = Tick.TL_BOTTOM;
			tickPosY = Tick.TL_LEFT;

			break;

		case CoordinateRect2D.AREA_RIGHT:
			// x-Achse in der Mitte mit Beschriftung unten
			// y-Achse links mit Beschriftung links
			pointXAxesStart.x = xmin;
			pointXAxesStart.y = 0;
			pointXAxesEnd.x = xmax;
			pointXAxesEnd.y = 0;
			pointYAxesStart.x = xmin;
			pointYAxesStart.y = ymin;
			pointYAxesEnd.x = xmin;
			pointYAxesEnd.y = ymax;
			pointAxesIntersect.x = xmin;
			pointAxesIntersect.y = 0;
			tickPosX = Tick.TL_BOTTOM;
			tickPosY = Tick.TL_LEFT;

			break;

		case CoordinateRect2D.AREA_LOWERRIGHT:
			// x-Achse oben mit Beschriftung oben
			// y-Achse in der Mitte mit Beschriftung links
			pointXAxesStart.x = xmin;
			pointXAxesStart.y = ymax;
			pointXAxesEnd.x = xmax;
			pointXAxesEnd.y = ymax;
			pointYAxesStart.x = xmin;
			pointYAxesStart.y = ymin;
			pointYAxesEnd.x = xmin;
			pointYAxesEnd.y = ymax;
			pointAxesIntersect.x = xmin;
			pointAxesIntersect.y = ymax;
			tickPosX = Tick.TL_BOTTOM;
			tickPosY = Tick.TL_LEFT;

			break;

		case CoordinateRect2D.AREA_UPPER:
			// x-Achse unten mit Beschriftung unten
			// y-Achse in der Mitte mit Beschriftung links
			pointXAxesStart.x = xmin;
			pointXAxesStart.y = ymin;
			pointXAxesEnd.x = xmax;
			pointXAxesEnd.y = ymin;
			pointYAxesStart.x = 0;
			pointYAxesStart.y = ymin;
			pointYAxesEnd.x = 0;
			pointYAxesEnd.y = ymax;
			pointAxesIntersect.x = 0;
			pointAxesIntersect.y = ymin;
			tickPosX = Tick.TL_BOTTOM;
			tickPosY = Tick.TL_LEFT;
			break;

		case CoordinateRect2D.AREA_ALL:
			// x-Achse in der Mitte mit Beschriftung unten
			// y-Achse in der Mitte mit Beschriftung links

			pointXAxesStart.x = xmin;
			pointXAxesStart.y = 0;
			pointXAxesEnd.x = xmax;
			pointXAxesEnd.y = 0;
			pointYAxesStart.x = 0;
			pointYAxesStart.y = ymin;
			pointYAxesEnd.x = 0;
			pointYAxesEnd.y = ymax;
			pointAxesIntersect.x = 0;
			pointAxesIntersect.y = 0;
			tickPosX = Tick.TL_BOTTOM;
			tickPosY = Tick.TL_LEFT;

			break;
		case CoordinateRect2D.AREA_LEFT:
			// x-Achse in der Mitte mit Beschriftung unten
			// y-Achse rechts mit Beschriftung rechts
			pointXAxesStart.x = xmin;
			pointXAxesStart.y = 0;
			pointXAxesEnd.x = xmax;
			pointXAxesEnd.y = 0;
			pointYAxesStart.x = xmax;
			pointYAxesStart.y = ymin;
			pointYAxesEnd.x = xmax;
			pointYAxesEnd.y = ymax;
			pointAxesIntersect.x = xmax;
			pointAxesIntersect.y = 0;
			tickPosX = Tick.TL_BOTTOM;
			tickPosY = Tick.TL_RIGHT;

			break;

		case CoordinateRect2D.AREA_UPPERLEFT:
			// x-Achse unten mit Beschriftung unten
			// y-Achse rechts mit Beschriftung rechts
			pointXAxesStart.x = xmin;
			pointXAxesStart.y = ymin;
			pointXAxesEnd.x = xmax;
			pointXAxesEnd.y = ymin;
			pointYAxesStart.x = xmax;
			pointYAxesStart.y = ymin;
			pointYAxesEnd.x = xmax;
			pointYAxesEnd.y = ymax;
			pointAxesIntersect.x = xmax;
			pointAxesIntersect.y = ymin;
			tickPosX = Tick.TL_BOTTOM;
			tickPosY = Tick.TL_RIGHT;
			break;

		case CoordinateRect2D.AREA_LOWERLEFT:
			// x-Achse oben mit Beschriftung oben
			// y-Achse links mit Beschriftung links
			pointXAxesStart.x = xmin;
			pointXAxesStart.y = ymax;
			pointXAxesEnd.x = xmax;
			pointXAxesEnd.y = ymax;
			pointYAxesStart.x = xmax;
			pointYAxesStart.y = ymin;
			pointYAxesEnd.x = xmax;
			pointYAxesEnd.y = ymax;
			pointAxesIntersect.x = xmax;
			pointAxesIntersect.y = ymax;
			tickPosX = Tick.TL_TOP;
			tickPosY = Tick.TL_RIGHT;

			break;
		case CoordinateRect2D.AREA_LOWER:
			// x-Achse oben mit Beschriftung oben
			// y-Achse in der Mitte mit Beschriftung links
			pointXAxesStart.x = xmin;
			pointXAxesStart.y = ymax;
			pointXAxesEnd.x = xmax;
			pointXAxesEnd.y = ymax;
			pointYAxesStart.x = 0;
			pointYAxesStart.y = ymin;
			pointYAxesEnd.x = 0;
			pointYAxesEnd.y = ymax;
			pointAxesIntersect.x = 0;
			pointAxesIntersect.y = ymax;
			tickPosX = Tick.TL_TOP;
			tickPosY = Tick.TL_LEFT;
			break;

		default:
			break;
		}

		// Falls eine der Achsen nicht sichtbar ist ist der Schnittpunkt leer
		if (!xAxisVisible() || !yAxisVisible()) {
			pointAxesIntersect = null;
		}

		// Falls x-Achse sichtbar x-Achse berechnen
		if (xAxisVisible()) {
			xaxis.compute(pointXAxesStart, pointXAxesEnd, pointAxesIntersect,
					tickPosX, (Graphics2D) panel.getGraphics(), this.scene2d);
		}

		// Falls y-Achse sichtbar x-Achse berechnen
		if (yAxisVisible()) {
			yaxis.compute(pointYAxesStart, pointYAxesEnd, pointAxesIntersect,
					tickPosY, (Graphics2D) panel.getGraphics(), this.scene2d);

		}
	}

	/** Setzt den Titel f?r die x-Achse */
	public void setCaptionXAxis(String label) {
		xaxis.caption = label;

	}

	/** Setzt den Titel f?r die y-Achse */
	public void setCaptionYAxis(String label) {
		yaxis.caption = label;

	}

	/**
	 * Berechnet die Positionen der Ticks auf einer Achse
	 * 
	 * @param min
	 *            minimaler Wert auf der Achse
	 * @param max
	 *            maximaler Wert auf der Achse
	 * @return Vector aus <CODE>TickPosition</CODE> Objekten
	 */
	Vector computeTickPositions(double min, double max) {

		// Vector f?r die Ticks
		Vector tickPos = new Vector();
		boolean bigTick;

		switch (style) {

		case MODE_INTELLIGENT:

			if (max > 0 && min < 0) {
				// falls 0 echt enthalten gehen von 0 weg zu den weiteren
				// Postionen
				tickPos.add(new TickPosition(0, true));

				double maximum = Math.abs(max) >= Math.abs(min) ? Math.abs(max)
						: Math.abs(min);

				double step = Rounder.roundDownToLeadingDec(Math.abs(maximum))

				/ (numTicks - 1);

				// Ticks von 0 bis max hinzuf?gen
				int i = 1;
				double pos = step;
				while (pos <= max + step / 2) {

					bigTick = i % 5 == 0 ? true : false;
					tickPos.add(new TickPosition(pos, bigTick));
					i++;
					pos = step * i;

				}

				// maximum noch hinzuf?gen
				bigTick = i % 5 == 0 ? true : false;
				tickPos.add(new TickPosition(max, bigTick));

				// Ticks von 0 bis Minimum hinzuf?gen
				pos = -step;
				i = 1;
				while (pos >= min - step / 2) {
					bigTick = i % 5 == 0 ? true : false;
					tickPos.add(new TickPosition(pos, bigTick));
					i++;
					pos = -step * i;
				}
				// Minimum hinzuf?gen
				bigTick = i % 5 == 0 ? true : false;
				tickPos.add(new TickPosition(min, bigTick));

			} else {

				if (max > 0 && min >= 0) {
					// nur positive Werte -> gehen von 0 weg

					tickPos.add(new TickPosition(min, true));

					double step = Rounder.roundDownToLeadingDec(Math.abs(max
							- min))

							/ (numTicks - 1);

					double pos = min + step;
					int i = 1;
					while (pos < max + step / 2) {
						bigTick = i % 5 == 0 ? true : false;
						tickPos.add(new TickPosition(pos, bigTick));
						i++;
						pos = min + step * i;

					}

				} else {

					double step = Rounder.roundDownToLeadingDec(Math.abs(max
							- min))
							/ (numTicks - 1);

					tickPos.add(new TickPosition(max, true));
					double pos = max - step;
					int i = 1;
					while (pos > min - step / 2) {
						bigTick = i % 5 == 0 ? true : false;
						tickPos.add(new TickPosition(pos, bigTick));
						i++;
						pos = max - step * i;
					}

				}
			}

			break;

		case MODE_BOX:
			// bei diesem Stil einfach von min bis max die Ticks einteilen

			// Minimum hinzuf?gen
			tickPos.add(new TickPosition(min, true));

			// Ticks zw. Minimum und Maximum hinzuf?gen
			double step = (max - min) / (numTicks - 1);
			double pos = min + step;
			int i = 1;
			while (pos < max - step / 2) {
				bigTick = i % 5 == 0 ? true : false;
				tickPos.add(new TickPosition(pos, bigTick));
				i++;
				pos = min + step * i;
			}
			// Maximum hinzuf?gen
			tickPos.add(new TickPosition(max, true));
			break;

		default:
			break;

		}

		return tickPos;
	}

	/**
	 * Erzeugt die Ticks
	 * 
	 * @param tickPositions
	 *            Vektor mit den Positionen der Tick
	 * @param tickDrawPos
	 *            Position der Ticks (TL_LEFT,...)
	 * @param roundingDiff
	 *            die zum Erzeugen der Beschriftung der Ticks erlaubte Rundung
	 * @param axisPos
	 *            Position der Achse (f?r x-Achse: y-Koordinate der Achse, f?r
	 *            y-Achse: x-Koordinate der Achse)
	 * @return <CODE>Vector</CODE> mit den Ticks
	 */
	Vector createTicks(Vector tickPositions, int tickDrawPos,
			double roundingDiff, double axisPos, DecimalFormat format) {

		// Vector f?r die Ticks
		Vector ticks = new Vector();

		for (int i = 0; i < tickPositions.size(); i++) {

			TickPosition t = (TickPosition) tickPositions.elementAt(i);

			// x-Koordinate des Ticks
			double posx = (tickDrawPos == Tick.TL_LEFT || tickDrawPos == Tick.TL_RIGHT) ? axisPos
					: t.position;

			// y-Koordinate des Ticks
			double posy = (tickDrawPos == Tick.TL_LEFT || tickDrawPos == Tick.TL_RIGHT) ? t.position
					: axisPos;

			Tick tick;

			if (t.maintick) {

				boolean add = true;

				// Standard f?r die Beschriftung ist zentrieren
				// bei Schnittpunkt der Achsen m?ssen die Beschriftung aber
				// verschoben werden

				int movedir = Tick.LABEL_CENTER;

				if (pointAxesIntersect != null) {

					if (area == CoordinateRect2D.AREA_ALL
							&& posy == pointAxesIntersect.y
							&& posx == pointAxesIntersect.x) {
						// der Tick ist Gerade der Schnittpunkt der Achsen
						// -> er wird nicht hinzugef?gt
						add = false;

					}

					if ((tickDrawPos == Tick.TL_LEFT || tickDrawPos == Tick.TL_RIGHT)
							&& posy == pointAxesIntersect.y) {
						movedir = moveLabel(tickDrawPos);
					}
					if ((tickDrawPos == Tick.TL_TOP || tickDrawPos == Tick.TL_BOTTOM)
							&& posx == pointAxesIntersect.x) {
						movedir = moveLabel(tickDrawPos);

					}

				}

				if (add) {

					String str;
					if (directiony != 3 && (tickDrawPos == Tick.TL_LEFT || tickDrawPos == Tick.TL_RIGHT)) 
						t.position=t.position==0?0:-t.position;
					if (directionx != 3 && (tickDrawPos == Tick.TL_TOP || tickDrawPos == Tick.TL_BOTTOM)) 
						t.position=t.position==0?0:-t.position;
					if (format == null) {
						str = formatter.format(t.position, roundingDiff);
					} else {
						str = format.format(t.position);
					}
					tick = new Tick(tickDrawPos, posx, posy, t.maintick, str,
							movedir, this);

					ticks.add(tick);

				}
			} else {
				// hier sind wir bei einem normalen kleinen Tick, der keine
				// Beschriftung hat
				tick = new Tick(tickDrawPos, posx, posy, t.maintick, null,
						Tick.LABEL_CENTER, this);

				ticks.add(tick);

			}
		}

		return ticks;
	}

	/**
	 * Gibt zur?ck wohin die Beschriftung eines Ticks (relativ zu dem Tick)
	 * hinverschoben werden muss (dies ist beim Schnittpunkt der Achsen
	 * notwendig)
	 */
	private int moveLabel(int tickDrawPos) {

		if (area == CoordinateRect2D.AREA_RIGHT
				&& tickDrawPos == Tick.TL_BOTTOM) {
			return Tick.LABEL_MOVERIGHT;
		}

		if (area == CoordinateRect2D.AREA_LOWERRIGHT
				&& tickDrawPos == Tick.TL_BOTTOM) {
			return Tick.LABEL_MOVERIGHT;

		}

		if (area == CoordinateRect2D.AREA_UPPER && tickDrawPos == Tick.TL_LEFT) {
			return Tick.LABEL_MOVEUP;
		}

		if (area == CoordinateRect2D.AREA_LOWER && tickDrawPos == Tick.TL_LEFT) {
			return Tick.LABEL_MOVEDOWN;
		}

		if (area == CoordinateRect2D.AREA_LEFT && tickDrawPos == Tick.TL_BOTTOM) {
			return Tick.LABEL_MOVELEFT;
		}

		return Tick.LABEL_CENTER;

	}

}