package uibk.mtk.draw2d.objects;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.image.BufferedImage;

import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.lang.PrepaintComputable;

/**
 * Klasse f?r einen Pfeil gegeben durch Fu?punkt und Kopfpunkt. Man kann die
 * Farbe und den Linienstil einstellen. Die Koordinaten sind Weltkoordinaten
 * (Koordinaten in einer <CODE>Scene2D</CODE>).
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public class MathArrow2D extends Drawable2D implements PrepaintComputable {

	/** Fu?punkt in Weltkoordinaten */
	protected Punkt2D base = new Punkt2D();

	/** Kopfpunkt in Weltkoordinaten */
	protected Punkt2D head = new Punkt2D();

	/** Fu?punkt in Ger?tekoordinaten */
	protected Point basepix = new Point(0, 0);

	/** Kopfpunkt in Ger?tekoordinaten */
	protected Point headpix = new Point(1, 0);

	/** Endpunkt des linken Fl?gels in Ger?tekoordinaten */
	protected Point leftwingpix = new Point();

	/** Endpunkt des rechten Fl?gels in Ger?tekoordinaten */
	protected Point rightwingpix = new Point();

	/** Stil zum Zeichnen der Linie */
	Stroke stroke = new BasicStroke();

	/** L?nge der Pfeilspitze */
	protected static final int LENGTHARROWHEAD = 6;

	/** Offset der Fl?gel (in Richtung normal auf die Pfeilrichtung) */
	protected static final int OFFSETWINGS = 3;

	/**
	 * Erzeugt einen Pfeil mit ?bergebenen Fu?punkt und Kopfpunkt
	 * 
	 * @param base
	 *            Fu?punkt
	 * @param head
	 *            Kopfpunkt
	 */
	public MathArrow2D(Punkt2D base, Punkt2D head) {
		this.base.setLocation(base);
		this.head.setLocation(head);
	}

	/** Erzeugt einen Pfeil mit Fu?punkt in (0,0) und Kopfpunkt in (1,0) */
	public MathArrow2D() {
	}

	/** setzt den Linienstil */
	public void setStroke(Stroke stroke) {
		this.stroke = stroke;
	}

	/** Gibt den Kopfpunkt zur?ck */
	public Punkt2D getHead() {
		return head;
	}

	/** Gibt den Fu?punkt zur?ck */
	public Punkt2D getBase() {
		return base;
	}

	/** Setzt den Fu?punkt */
	public void setBase(Punkt2D base) {
		this.base.setLocation(base);

	}

	/** Setzt den Kopfpunkt */
	public void setHead(Punkt2D head) {
		this.head.setLocation(head);

	}

	/** Zeichnet den Pfeil */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {

		if (!draw)
			return;

		//Farbe und Linienstil setzen
		Stroke old = g2.getStroke();
		g2.setStroke(stroke);
		g2.setColor(color);

		g2.drawLine(basepix.x, basepix.y, headpix.x, headpix.y);
		g2.drawLine(rightwingpix.x, rightwingpix.y, headpix.x, headpix.y);
		g2.drawLine(leftwingpix.x, leftwingpix.y, headpix.x, headpix.y);

		g2.setStroke(old);

	}

	/**
	 * Berechnet den Pixelkoordinaten f?r den Fu?punkt, den Kopfpunkt und die
	 * Endpunkte der "Fl?gel" Falls dabei ?berlauf/Div. durch 0 auftritt wird
	 * <CODE>draw</CODE> auf false gesetzt und der Pfeil somit nicht
	 * gezeichnet.
	 */
	public void prepaintcompute() throws Exception {
		draw = false;
		basepix = scene2d.project(base);
		headpix = scene2d.project(head);

		Punkt2D dir = head.sub(base);

		//Einheitsvektor erzeugen
		dir.normalize();

		if (Double.isNaN(dir.x) || Double.isInfinite(dir.x)
				|| Double.isNaN(dir.y) || Double.isInfinite(dir.y)
				|| (dir.x == 0 && dir.y == 0)) {
			draw = false;

			return;
		}

		//Skalieren wichtig damit dirPix nicht (0,0) wird
		double s = Math.max(scene2d.getXRange(), scene2d.getYRange()) / 10;
		dir.scaleself(s);

		Punkt2D dirPix;
		Punkt2D zero = scene2d.projectdouble(new Punkt2D(0, 0));
		dirPix = (scene2d.projectdouble(dir)).sub(zero);

		dirPix.normalize();
		if (Double.isNaN(dirPix.x) || Double.isInfinite(dirPix.x)
				|| Double.isNaN(dirPix.y) || Double.isInfinite(dirPix.y)
				|| (dirPix.x == 0 && dirPix.y == 0)) {
			draw = false;
			return;
		}

		Punkt2D dirnormalPix = new Punkt2D(-dirPix.y, dirPix.x);

		leftwingpix.x = (int) (headpix.x - LENGTHARROWHEAD * dirPix.x + OFFSETWINGS
				* dirnormalPix.x);
		leftwingpix.y = (int) (headpix.y - LENGTHARROWHEAD * dirPix.y + OFFSETWINGS
				* dirnormalPix.y);
		rightwingpix.x = (int) (headpix.x - LENGTHARROWHEAD * dirPix.x - OFFSETWINGS
				* dirnormalPix.x);
		rightwingpix.y = (int) (headpix.y - LENGTHARROWHEAD * dirPix.y - OFFSETWINGS
				* dirnormalPix.y);

		draw = true;

	}

}