package uibk.mtk.draw2d.objects;

import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.awt.BasicStroke;

import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.lang.ToolTipable;

/**
 * Eine 2 dimensionaler Bezugspunkt mit vernderbarer Gre, Farbe und Tooltips .
 * <p>
 * Die Koordinaten sind Weltkoordinaten in einer <CODE>Scene2D</CODE>. Falls
 * kein Tooltip festgesetzt wird und der Tooltip aktiviert ist werden die
 * aktuellen Weltkoordinaten in der Form ( <i>x-Koordinate </i>, <i>y-Koordinate
 * </i>) als Tooltip verwendet
 * <p>
 * 
 * <p>
 * Copyright: Institut fr Technische Mathematik, Geometrie und Bauinformatik,
 * 2006
 * </p>
 * 
 * @author Christian Reisecker (christian.reisecker@gmx.at)
 * 
 */
public class PhysicBaseWall extends Drawable2D implements ToolTipable {

	/** x-Koordinate des Bezugspunkt im aktuellen Koordinatensystem */
	private double x=0;

	/** x-Koordinate des Bezugspunkt im aktuellen Koordinatensystem */
	private double y=0;

	/** Gre des Bezugspunktes (in Pixeln x-Koordinate) */
	private int sizeX = 5;

	/** Gre des Bezugspunktes (in Pixeln y-Koordinate) */
	private int sizeY = sizeX / 3;

	/** Tooltips aktiviert oder nicht */
	private boolean tooltipenabled = false;

	/** Tooltip-Text (Tooltips mssen aktiviert sein) */
	private String toolTip = null;

	/** Ausrichtung des Bezugspunkts nach Norden */
	public static final int NORTH = 0;

	/** Ausrichtung des Bezugspunkts nach Osten */
	public static final int EAST = 1;

	/** Ausrichtung des Bezugspunkts nach Sden */
	public static final int SOUTH = 2;

	/** Ausrichtung des Bezugspunkts nach Westen */
	public static final int WEST = 3;

	/** Ausrichtung des Bezugspunkts */
	private int orientation = SOUTH;

	/** default Konstruktor (Punkt=(0,0))* */
	public PhysicBaseWall() {
	}

	/**
	 * Konstruktor der einen Bezugspunkt mit angegebenen Koordinaten anlegt
	 * 
	 * @param x
	 *            x-Weltkoordinate
	 * @param y
	 *            y-Weltkoordinate
	 */
	public PhysicBaseWall(double x, double y) {

		this.x = x;
		this.y = y;

	}

	/**
	 * Gibt x-Koordinate zurck
	 * 
	 * @return x x-Weltkoordinate
	 */
	public double getX() {
		return x;

	}

	/**
	 * Gibt y-Koordinate zurck
	 * 
	 * @return y y-Weltkoordinate
	 */
	public double getY() {
		return y;

	}

	/**
	 * Setzt die x-Koordinate des Bezugspunkt
	 * 
	 * @param x
	 *            x-Weltkoordinate
	 */
	public void setX(double x) {
		this.x = x;

	}

	/**
	 * Setzt die y-Koordinate des Bezugspunkt
	 * 
	 * @param y
	 *            y-Weltkoordinate
	 */
	public void setY(double y) {
		this.y = y;
	}

	/**
	 * setzt die Koordinaten des Bezugspunkt
	 * 
	 * @param x
	 *            x-Weltkoordinate
	 * @param y
	 *            y-Weltkoordinate
	 */
	public void setCoordinates(double x, double y) {
		this.x = x;
		this.y = y;
	}

	/** Setzt den Text der als Tooltip verwendet werden soll */
	public void setToolTipText(String tooltip) {
		this.toolTip = tooltip;
	}

	/** Setzt die Koordinaten des Punktes auf die Koordinaten des Arguments
	 * 
	 * @param p Punkt in Weltkoordinaten
	 */
	public void setCoordinates(Punkt2D p) {
		this.x = p.x;
		this.y = p.y;

	}

	/** Gibt zurck ob der Tooltip aktiviert ist oder nicht */
	public boolean isToolTipEnabled() {
		return this.tooltipenabled;
	}

	/** Aktiviert oder deaktiviert den Tooltip */
	public void enableToolTip(boolean b) {
		this.tooltipenabled = b;
	}

	/**
	 * Setzt die Gre des Bezugspunkt zum Zeichnen fest
	 * 
	 * @param size
	 *            Gre des Bezugspunktes in Pixeln
	 */
	public void setSize(int size) {
		this.sizeX = size;
		this.sizeY = size / 3;
	}

	/** gibt zurck, ob die Gertekoordinaten ber dem Bezugspunkt sind
	 * 
	 * @param xm
	 *            x-Weltkoordinate
	 * @param ym
	 *            y-Weltkoordinate
	 */
	public boolean tooltipcontains(int xm, int ym) {

		int xp = super.scene2d.xToPixel(x);
		int yp = super.scene2d.yToPixel(y);

		if ((xm - xp) * (xm - xp) + (ym - yp) * (ym - yp) > sizeX * sizeY) {
			return false;
		}
		return true;

	}

	/** gibt die Koordinaten des Punktes zurck
	 * 
	 * @return Punkt in Weltkoordinaten
	 */
	public Punkt2D getPunkt2D() {
		return new Punkt2D(x, y);
	}

	/**
	 * Gibt den Tooltip des Punktes zurck, falls keiner gesetzt wurde,d.h.
	 * falls Tooltip==null, wird "(x,y)" zurckgegeben
	 */
	public String getToolTipText(MouseEvent ev) {

		if (this.toolTip == null) {
			String str = "("
					+ String.valueOf(x) + ","
					+ String.valueOf(y) + ")";
			return (str);
		} else {
			return toolTip;
		}
	}

	/**
	 * Zeichnen des Bezugspunkt der gesetzten Gre und gesetzter Farbe
	 */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {

		int tmpSizeX = sizeX;
		int tmpSizeY = sizeY;

		g2.setColor(color);

		g2.setStroke(new BasicStroke((sizeX >= 0) ? sizeX / 20 : -sizeX / 20));
		switch (orientation) {

		// Norden
		case 0:
			tmpSizeX = sizeY;
			tmpSizeY = sizeX;
			tmpSizeX = -tmpSizeX;
		// Sden
		case 2:
			if (orientation != 0) {
				tmpSizeX = sizeY;
				tmpSizeY = sizeX;
				}
			g2.drawLine(scene2d.xToPixel(x),
					scene2d.yToPixel(y) - tmpSizeY /2,
					scene2d.xToPixel(x),
					scene2d.yToPixel(y) + tmpSizeY / 2);
		
			g2.setStroke(new BasicStroke((tmpSizeX >= 0) ? tmpSizeX / 30
					: -tmpSizeX / 30));

			int abstandy = tmpSizeY / 5;
			for (int i = 0; i < 6; i++)
				g2.drawLine(scene2d.xToPixel(x),scene2d.yToPixel(y) - tmpSizeY / 2 + i * abstandy,
						scene2d.xToPixel(x)- tmpSizeX
						,scene2d.yToPixel(y) - tmpSizeY / 2
						+ i * abstandy + tmpSizeY / 5);
			break;

		// Osten
		case 1:
			tmpSizeX = -tmpSizeX;
		// Westen
		case 3:
			g2.drawLine(scene2d.xToPixel(x) - tmpSizeX /2,
					scene2d.yToPixel(y), scene2d.xToPixel(x) + tmpSizeX / 2,
					scene2d.yToPixel(y));
			g2.setStroke(new BasicStroke((tmpSizeX >= 0) ? tmpSizeX / 30
					: -tmpSizeX / 30));

			int abstandx = tmpSizeX / 5;
			for (int i = 0; i < 6; i++)
				g2.drawLine(scene2d.xToPixel(x) - tmpSizeX / 2 + i * abstandx,
						scene2d.yToPixel(y), scene2d.xToPixel(x) - tmpSizeX / 2
								+ i * abstandx + tmpSizeX / 5, scene2d
								.yToPixel(y)- tmpSizeY);
				}

	}

	/**
	 * ndern der Richtung zur Auswahl stehen <CODE> NORTH, EAST, SOUTH, WEST
	 * <CODE>
	 */
	public void setOrientation(int o) {
		orientation = o;
	}
}
