package uibk.mtk.math;

/**
 * 
 * Utilityklasse zum Runden von Gleitkommazahlen
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik
 * der Universit?t Innsbruck, 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 *  
 */

public final class Rounder {

	private Rounder() {

	}

	/** Rundungsmodus HALF_UP */
	private static final int ROUND_HALF_UP = 1;

	/** Rundungsmodus FLOOR */
	private static final int ROUND_FLOOR = 2;

	/** Rundungsmodus CEIL */
	private static final int ROUND_CEIL = 3;

	/** Rundungsmodus DOWN */
	private static final int ROUND_DOWN = 4;

	/** Rundungsmodus */
	private static int roundingmode = ROUND_HALF_UP;

	/**
	 * Rundet die ?bergebene Gleitkommazahl zur n?chsten Gleitkommazahl wobei
	 * mit der durch <CODE>roundingmode</CODE> festgelegten Rundungsart
	 * gerundet wird
	 */
	private static synchronized double round(double x) {
		switch (roundingmode) {
		case ROUND_HALF_UP:
			return Math.floor(x + 0.5d);

		case ROUND_FLOOR:
			return Math.floor(x);
		case ROUND_CEIL:
			return Math.ceil(x);

		case ROUND_DOWN:
			if (x >= 0)
				return Math.floor(x);
			else
				return -Math.floor(-x);

		default:
			throw new IllegalArgumentException(""); //$NON-NLS-1$
		}
	}

	/**
	 * Runden einer ganzen Zahl
	 * 
	 * @param x
	 *            eine ganze Zahl
	 * @param distance
	 *            max. Abstand der gerundeten Zahl zum Ausgangswert <CODE>x
	 *            </CODE>
	 *  
	 */
	private static synchronized double roundentire(double x, double distance) {

		double r = x;

		if (r == 0)
			return 0;

		double temp;

		int c = 1;

		do {
			temp = r;

			r = round(r / Math.pow(10, c)) * Math.pow(10, c);

			if (r == 0) {
				if ((Math.abs(x) <= distance))
					return 0;
				else
					return temp;

			}

			c++;

		} while (Math.abs(r - x) <= distance);

		return temp;

	}

	/**
	 * Runden der Nachkommastellen einer Gleitkommazahl
	 * 
	 * @param x
	 *            die Zahl die gerundet werden soll
	 * @param distance
	 *            max. Abstand der gerundeten Zahl zum Ausgangswert <CODE>x
	 *            </CODE>
	 *  
	 */
	private static synchronized double roundplaces(double x, double distance) {
		int i = 1;
		double r = x;

		do {
			r = roundNumPlaces(x, i);
			i++;
		} while (Math.abs(r - x) >= distance && i < 15);

		return r;
	}

	/**
	 * Runden der einer Gleitkommazahl
	 * 
	 * @param x
	 *            die Zahl die gerundet werden soll
	 * @param distance
	 *            max. Abstand der gerundeten Zahl zum Ausgangswert <CODE>x
	 *            </CODE>
	 *  
	 */
	private static synchronized double roundit(double x, double distance) {
		if (Double.isInfinite(distance) && Double.isNaN(distance)
				|| Double.isInfinite(x) && Double.isNaN(x)) {
			return x;
		}
		double r = round(x);

		if (Math.abs(r - x) <= distance) {
			//wir haben eine ganze Zahl und wir runden diese

			return roundentire(r, distance);

		} else {

			return roundplaces(x, distance);
		}

	}

	/**
	 * Gibt einen m?glichst gerundeten Wert der Gleitkommazahl x zur?ck, wobei
	 * der gerundete Wert h?chstens distance von x entfernt ist. Der vorgegebene
	 * Abstand <CODE>distance</CODE> wird dabei so weit ausgenutzt wie m?glich
	 * ausgenutzt. Der zur?ckgebene Wert ist wegen der auftretenden
	 * Rundungsfehler/Darstellungsfehler nicht 100% rund (z.B.
	 * 7.1*1E132=7.099999999999999E132)
	 *  
	 */
	public static synchronized double getRoundValue(double x, double distance) {

		roundingmode = ROUND_HALF_UP;
		return roundit(x, Math.abs(distance));

	}

	/**
	 * Gibt einen m?glichst gerundeten (nach - Unendlich) Wert der
	 * Gleitkommazahl x zur?ck, wobei der gerundete Wert h?chstens distance von
	 * x entfernt ist. Der vorgegebene Abstand <CODE>distance</CODE> wird
	 * dabei so weit ausgenutzt wie m?glich ausgenutzt. Der zur?ckgebene Wert
	 * ist wegen der auftretenden Rundungsfehler/Darstellungsfehler nicht 100%
	 * rund.
	 */
	public static double getLowerRoundValue(double x, double distance) {

		double r = getUpperRoundValue(-x, distance);
		if (r != 0)
			return -r;
		else
			return 0;

	}

	/**
	 * Gibt einen m?glichst gerundeten (nach + Unendlich) Wert der
	 * Gleitkommazahl x zur?ck, wobei der gerundete Wert h?chstens distance von
	 * x entfernt ist. Der vorgegebene Abstand <CODE>distance</CODE> wird
	 * dabei so weit ausgenutzt wie m?glich ausgenutzt. Der zur?ckgebene Wert
	 * ist wegen der auftretenden Rundungsfehler/Darstellungsfehler nicht 100%
	 * rund.
	 */
	public static double getUpperRoundValue(double x, double distance) {
		roundingmode = ROUND_CEIL;
		return roundit(x, distance);

	}

	/**
	 * Gibt eine im Betrag auf die f?hrende (positive oder negative)
	 * Zehnerpotenz Richtung Null gerundete Zahl des Arguments zur?ck. z.b:
	 * 13.322->10.000 und 0.00534->0.005 Die gerundete Zahl ist somit immer
	 * weniger als 1/2 der Gr?sse des Arguments vom Argument entfernt (worst
	 * case: z.B. 1999->1000) <br>
	 * Falls bei der Berechnung ein ?berlauf auftritt wird der urspr?ngliche
	 * Wert zur?ckgegeben
	 */
	public static synchronized double roundDownToLeadingDec(double x) {

		int fac = 1;

		double r, ret;
		r = Math.abs(x);

		if (r > 1) {

			while (Math.floor(r / fac) > 0) {
				fac = fac * 10;
			}
			fac = fac / 10;

			ret = Math.floor(r / fac) * fac;

		} else {
			while (Math.floor(r * fac) == 0) {
				fac = fac * 10;
			}
			ret = Math.floor(r * fac) / fac;

		}

		if (x < 0) {
			return -ret;
		} else {
			return ret;
		}
	}

	/** Rundet eine Zahl auf die angebene Zahl von Stellen */
	public static synchronized double roundNumPlaces(double x, int places) {
		long factor = (long) Math.pow(10, places);
		x = x * factor;
		double tmp = round(x);
		return tmp / factor;
	}

}