package uibk.mtk.parser.realparser;

import java.util.Hashtable;

import uibk.mtk.lang.Messages;

/**
 * Parser f?r reelle mathematische Ausdr?cke. *
 * <p>
 * Diese Klasse ist dem frei verwendbaren Packet <i>expr </i> von Darius Bacon
 * (siehe http://www.accesscom.com/~darius/) entnommen und wurde ge?ndert.
 * </p>
 * 
 * 
 * 
 * 
 * <p>
 * Die folgenden Operatoren, in absteigender Folge ihrer Priorit?t, sind:
 * 
 * <UL>
 * <LI>^ (hoch) /
 * <LI>un?res Minus (-x)
 * <LI>+ -
 * <LI>&lt; &lt;= = &lt;&gt; &gt;= &gt;
 * <LI>and (logisches Und)
 * <LI>or (logisches Oder)
 * </UL>^ bindet von rechts nach links, die anderen Operatoren von links nach
 * rechts
 * 
 * </P>
 * 
 * <P>
 * Die bekannten un?ren Funktionen abs, acos, asin, atan, ceil, cos, exp, floor,
 * log, round, sin, sqrt, tan. Jede dieser Fkt. erwartet ihr Argument in
 * Klammern.
 * </P>
 * <P>
 * Die bekannten bin?ren Fkt.: atan2, min, max; und eine tern?re
 * Bedingungsklausel if(test, then, else). *
 * </P>
 * <p>
 * Examples:
 * <UL>
 * <LI>42
 * <LI>2-3
 * <LI>cos(x^2) + sin(x^2)
 * </ul>
 * </p>
 */
public class Parser {

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.parser.realparser.messages";//$NON-NLS-1$

	/** die "registrierten" Variablen */
	private Hashtable variables = new Hashtable();

	/** Scanner zum Zerlegen der Eingabe in Token */
	Scanner tokenscanner = null;

	/** aktueller Token */
	private Token token = null;

	/** die Eingabe */
	String input;

	/** die Operatoren */
	private String operatorChars = "*/+-^<>=,()!"; //$NON-NLS-1$

	/** die un?ren Funktionen */
	private static final String[] PROCS1 = {
			"abs", "acos", "asin", "atan", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			"ceil", "cos", "exp", "floor", "log", "round", "sin", "sqrt", "tan", "faktorielle", "sign" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-10$ //$NON-NLS-11$

	/** die Codes f?r die un?ren Operatoren */
	private static final int[] RATORS1 = { Expr.ABS, Expr.ACOS, Expr.ASIN,
			Expr.ATAN, Expr.CEIL, Expr.COS, Expr.EXP, Expr.FLOOR, Expr.LOG,
			Expr.ROUND, Expr.SIN, Expr.SQRT, Expr.TAN, Expr.FACTORIAL,
			Expr.SIGNUM };

	/** die bin?ren Funktionen */
	private static final String[] PROCS2 = { "atan2", "max", "min" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

	/** die Codes f?r die bin?ren Operatoren */
	private static final int[] RATORS2 = { Expr.ATAN2, Expr.MAX, Expr.MIN };

	public Parser() {
		addStandardVariables();
	}

	/** L?schen aller vom Benutzer hinzugef?gter Variablen */
	public void clearVariables() {
		if (this.variables != null) {
			this.variables.clear();
			addStandardVariables();
		}

	}

	private void addStandardVariables() {
		Variable pi = new Variable("pi"); //$NON-NLS-1$
		pi.setValue(Math.PI);
		addVariable(pi);
	}

	public synchronized void addVariable(Variable var) {
		Variable result = (Variable) variables.get(var.name);
		if (result == null)
			variables.put(var.name, var);

	}

	private String trim(String input) {
		String str = ""; //$NON-NLS-1$
		int n = input.length();
		for (int i = 0; i < n; i++) {
			if (!Character.isWhitespace(input.charAt(i)))
				str += input.charAt(i);
		}

		return str;

	}

	/**
	 * Gibt die durch die Eingabe definierten Ausdrick zur?ck
	 * 
	 * @param in
	 *            die Eingabe
	 * 
	 * @exception SyntaxException
	 *                falls die Eingabe nicht parsbar ist
	 */
	public Expr parseString(String in) throws SyntaxException {
		this.input = trim(in);

		//Zerlegen des Ausdrucks in Token
		tokenscanner = new Scanner(input, operatorChars, this.variables);
		//parsen
		return reparse();

	}

	private Expr reparse() throws SyntaxException {
		tokenscanner.index = -1;
		nextToken();
		Expr expr = parseExpr(0);
		expr.formula = input;
		if (token.ttype != Token.TT_EOF)
			throw error(Messages.getString(BUNDLE_NAME, "Parser.0"), //$NON-NLS-1$
					SyntaxException.INCOMPLETE, null);

		return expr;
	}

	/** Gibt das n?chste Token zur?ck */
	private void nextToken() {
		token = tokenscanner.nextToken();
	}

	private Expr parseExpr(int precedence) throws SyntaxException {
		Expr expr = parseFactor();

		if (token.ttype == '!') {
			expr = Expr.makePost(Expr.FACTORIAL, expr);
			nextToken();

		}

		loop: for (;;) {
			int l, r, rator;

			// The operator precedence table.
			// l = left precedence, r = right precedence, rator = operator.
			// Higher precedence values mean tighter binding of arguments.
			// To associate left-to-right, let r = l+1;
			// to associate right-to-left, let r = l.

			switch (token.ttype) {

			case '<':
				l = 20;
				r = 21;
				rator = Expr.LT;
				break;
			case Token.TT_LE:
				l = 20;
				r = 21;
				rator = Expr.LE;
				break;
			case '=':
				l = 20;
				r = 21;
				rator = Expr.EQ;
				break;
			case Token.TT_NE:
				l = 20;
				r = 21;
				rator = Expr.NE;
				break;
			case Token.TT_GE:
				l = 20;
				r = 21;
				rator = Expr.GE;
				break;
			case '>':
				l = 20;
				r = 21;
				rator = Expr.GT;
				break;

			case '+':
				l = 30;
				r = 31;
				rator = Expr.ADD;
				break;
			case '-':
				l = 30;
				r = 31;
				rator = Expr.SUB;
				break;

			case '/':
				l = 40;
				r = 41;
				rator = Expr.DIV;
				break;
			case '*':
				l = 40;
				r = 41;
				rator = Expr.MUL;
				break;

			case '^':
				l = 50;
				r = 50;
				rator = Expr.POW;
				break;

			default:
				if (token.ttype == Token.TT_WORD && token.sval.equals("and")) { //$NON-NLS-1$
					l = 5;
					r = 6;
					rator = Expr.AND;
					break;
				}
				if (token.ttype == Token.TT_WORD && token.sval.equals("or")) { //$NON-NLS-1$
					l = 10;
					r = 11;
					rator = Expr.OR;
					break;
				}

				break loop;
			}

			if (l < precedence)
				break loop;

			nextToken();
			expr = Expr.makeApp2(rator, expr, parseExpr(r));
		}
		return expr;
	}

	/** Erzeugt aus dem aktuellen Token einen Ausdruck */
	private Expr parseFactor() throws SyntaxException {
		switch (token.ttype) {
		case Token.TT_NUMBER:
			//falls das Token eine Zahl ist->konstanter Ausdruck
			Expr lit = Expr.makeLiteral(token.nval);
			nextToken();
			return lit;

		case Token.TT_WORD:
			//falls das Token ein Wort ist

			//haben wir einen un?ren Operator
			for (int i = 0; i < PROCS1.length; ++i)
				if (PROCS1[i].equals(token.sval)) {
					nextToken();
					expect('(');
					Expr rand = parseExpr(0);
					expect(')');
					return Expr.makeApp1(RATORS1[i], rand);
				}

			//haben wir einen bin?ren Operator
			for (int i = 0; i < PROCS2.length; ++i)
				if (PROCS2[i].equals(token.sval)) {
					nextToken();
					expect('(');
					Expr rand1 = parseExpr(0);
					expect(',');
					Expr rand2 = parseExpr(0);
					expect(')');
					return Expr.makeApp2(RATORS2[i], rand1, rand2);
				}

			//haben wir eine if-Bedingung
			if (token.sval.equals("if")) { //$NON-NLS-1$
				nextToken();
				expect('(');
				Expr test = parseExpr(0);
				expect(',');
				Expr consequent = parseExpr(0);
				expect(',');
				Expr alternative = parseExpr(0);
				expect(')');
				return Expr.makeIfThenElse(test, consequent, alternative);
			}

			//haben wir eine Variable
			Expr var = (Expr) variables.get(token.sval);

			//wir haben einen Unbekannten Ausdruck
			if (null == variables || null == variables.get(token.sval))
				throw error(
						Messages.getString(BUNDLE_NAME, "Parser.25") + ": " + token.sval, //$NON-NLS-1$
						SyntaxException.UNKNOWN_VARIABLE, null);

			//zum n?chsten Token gehen
			nextToken();
			return var;

		case '(':

			nextToken();
			Expr enclosed = parseExpr(0);
			expect(')');
			return enclosed;

		case '-':
			nextToken();
			return Expr.makeApp1(Expr.NEG, parseExpr(35));

		case Token.TT_EOF:
			throw error(Messages.getString(BUNDLE_NAME, "Parser.26"), //$NON-NLS-1$
					SyntaxException.PREMATURE_EOF, null);
		default:
			throw error(
					Messages.getString(BUNDLE_NAME, "Parser.27"), SyntaxException.BAD_FACTOR, null); //$NON-NLS-1$
		}
	}

	private SyntaxException error(String complaint, int reason, String expected) {
		return new SyntaxException(complaint, reason, expected);
	}

	private void expect(int ttype) throws SyntaxException {
		if (token.ttype != ttype)
			throw error(
					"'"		+ (char) ttype + "' " + Messages.getString(BUNDLE_NAME, "Parser.1"), //$NON-NLS-1$ //$NON-NLS-2$
					SyntaxException.EXPECTED, "" + (char) ttype); //$NON-NLS-1$
		nextToken();
	}

}